/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.work;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.zfabrik.work.IThreadPool;
import com.zfabrik.work.WorkManager;

public class WorkManagerImpl extends WorkManager {
	private WorkManagerImpl() {
	}

	public static void initialize() {
		if (WorkManager.instance == null) {
			WorkManager.instance = new WorkManagerImpl();
			ApplicationThreadPoolImpl.initialize();
		}
	}
	
	public static void shutdown() {
		WorkManagerImpl i = (WorkManagerImpl) WorkManager.instance;
		try {
			if (i!=null) {
				Set<String> tps = new HashSet<String>();
				while (true) {
					tps.clear();
					synchronized (i.threadPools) {
						tps.addAll(i.threadPools.keySet());
						if (tps.isEmpty()) {
							break;
						}
					}
					for (String s : tps) {
						try {
							i.releaseThreadPool(s);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				} 
			}
		} finally {
			WorkManager.instance=null;
		}
	}

	@Override
	public IThreadPool createThreadPool(String id) {
		synchronized (threadPools) {
			if (this.threadPools.keySet().contains(id))
				throw new IllegalArgumentException("thread pool \"" + id + "\" does already exist");
			ThreadPoolImpl tp = new ThreadPoolImpl(id);
			this.threadPools.put(id, tp);
			return tp;
		}
	}

	@Override
	public void releaseThreadPool(String id) {
		synchronized (threadPools) {
			ThreadPoolImpl tp = this.threadPools.remove(id);
			if (tp != null && !tp.isClosed()) {
				tp.close();
			}
		}
	}

	//
	//
	private Map<String, ThreadPoolImpl> threadPools = new HashMap<String, ThreadPoolImpl>();

}
