package com.zfabrik.impl.work.jmx;

import java.util.Date;

import com.zfabrik.impl.work.ThreadPoolImpl;

/**
 * JMX exposure
 */
public  class ThreadPoolStatistics implements ThreadPoolStatisticsMBean {

	private ThreadPoolImpl tp;
	private Object lock;
	
	public ThreadPoolStatistics(Object lock, ThreadPoolImpl tp) {
		this.tp = tp;
		this.lock = lock;
	}
	
	@Override
	public Date getLastResetTime() {
		synchronized (lock) { return tp.lastResetTime; }
	}

	@Override
	public int getMaxConcurrencyReached() {
		synchronized (lock) { return tp.maxConcurrencyReached; }
	}

	@Override
	public int getMaxConcurrencyAllowed() {
		synchronized (lock) { return tp.maxConcurrencyAllowed; }
	}

	@Override
	public int getConcurrency() {
		synchronized (lock) { return tp.concurrency; }
	}

	@Override
	public Date getLastDispatch() {
		synchronized (lock) {
			if (tp.lastDispatch>0) {
				return new Date(tp.lastDispatch); 
			} else {
				return null;
			}
		}
	}

	@Override
	public int getMaxQueueLength() {
		synchronized (lock) { return tp.maxTaskQueueLength; }
	}

	@Override
	public int getQueueLength() {
		synchronized (lock) { return tp.queue.size(); }
	}
	
	@Override
	public long getTasksCompleted() {
		synchronized (lock) { return tp.tasksCompleted; }
	}

	@Override
	public void reset() {
		synchronized (lock) {
			tp.maxConcurrencyReached = tp.concurrency;
			tp.maxTaskQueueLength = tp.queue.size();
			tp.lastResetTime=new Date();
			tp.tasksCompleted =0;		
		}
	}
} // 