/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.dev.z2jupiter.internal.transfer;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import org.junit.platform.engine.ConfigurationParameters;

/**
 * Trivial map based config params that we really just implement to 
 * satisfy the API requirements. On the client, we do not really
 * care.
 */
public class Z2ConfigurationParametersDto implements ConfigurationParameters, Serializable {
	private static final long serialVersionUID = 1L;
	private Map<String,String> m = new HashMap<>();

	public Z2ConfigurationParametersDto() {}

	public Z2ConfigurationParametersDto(ConfigurationParameters p) {
		p.keySet().forEach(k->p.get(k).ifPresent(v->m.put(k, v)));
	}

	@Override
	public Optional<String> get(String key) {
		return Optional.ofNullable(m.get(key));
	}

	@Override
	public Optional<Boolean> getBoolean(String key) {
		return get(key).map(Boolean::parseBoolean);
	}

	@Override
	public int size() {
		return m.size();
	}

	@Override
	public Set<String> keySet() {
		return m.keySet();
	}

}
